# Generic result codes
CMD_RESULT_OK = 0xC3
CMD_RESULT_FAIL = 0x3C
CMD_RESULT_UNAUTHORIZED = 0x01
CMD_RESULT_INVALID_CMD = 0x02

# Resutls for memory commands
CMD_RESULT_MEM_WRITE_FAIL = 0x10
CMD_RESULT_MEM_SLOT_EXPIRED = 0x11

# Results for ECC commands
CMD_RESULT_ECC_INVALID_KEY  = 0x12

# Results for MCounter
CMD_RESULT_MCOUNTER_UPDATE_ERROR = 0x13
CMD_RESULT_MCOUNTER_COUNTER_INVALID = 0x14

# Results for pairing keys
CMD_RESULT_PAIRING_KEY_EMPTY = 0x15
CMD_RESULT_PAIRING_KEY_INVALID = 0x16
