GET_INFO_X509_CERT = 0x00

# Get info Byte 0 
GET_INFO_CHIPID = 0x01
GET_INFO_RISCV_FW_VERSION = 0x02
GET_INFO_SPECT_FW_VERSION = 0x04
GET_INFO_FW_BANK = 0xB0

# Get info Byte 1
GET_INFO_DATA_CHUNK_0_127 = 0x00

# This is need onlt for GET_INFO_X509_CERT
GET_INFO_DATA_CHUNK_128_255 = 0x01
GET_INFO_DATA_CHUNK_256_383 = 0x02
GET_INFO_DATA_CHUNK_384_511 = 0x03
