"""
TropicSquare Factory Pairing Keys
=================================

This module defines the factory pairing keys used for TropicSquare chips.
It includes predefined keys for engineering samples and production units.

Main exports:

    - FACTORY_PAIRING_KEY_INDEX: Default key index for factory pairing
    - FACTORY_PAIRING_PRIVATE_KEY_ENG_SAMPLE: Private key for engineering samples
    - FACTORY_PAIRING_PUBLIC_KEY_ENG_SAMPLE: Public key for engineering samples
    - FACTORY_PAIRING_PRIVATE_KEY_PROD0: Private key for production unit 0
    - FACTORY_PAIRING_PUBLIC_KEY_PROD0: Public key for production unit 0
"""

# Factory pairing key index (slot 0)
FACTORY_PAIRING_KEY_INDEX = 0x00

# Engineering sample
FACTORY_PAIRING_PRIVATE_KEY_ENG_SAMPLE = bytes([
    0xd0,0x99,0x92,0xb1,0xf1,0x7a,0xbc,0x4d,0xb9,0x37,0x17,0x68,0xa2,0x7d,0xa0,0x5b,
    0x18,0xfa,0xb8,0x56,0x13,0xa7,0x84,0x2c,0xa6,0x4c,0x79,0x10,0xf2,0x2e,0x71,0x6b
])

FACTORY_PAIRING_PUBLIC_KEY_ENG_SAMPLE = bytes([
    0xe7,0xf7,0x35,0xba,0x19,0xa3,0x3f,0xd6,0x73,0x23,0xab,0x37,0x26,0x2d,0xe5,0x36,
    0x08,0xca,0x57,0x85,0x76,0x53,0x43,0x52,0xe1,0x8f,0x64,0xe6,0x13,0xd3,0x8d,0x54
])

# Production 0
FACTORY_PAIRING_PRIVATE_KEY_PROD0 = bytes([
    0x28,0x3F,0x5A,0x0F,0xFC,0x41,0xCF,0x50,0x98,0xA8,0xE1,0x7D,0xB6,0x37,0x2C,0x3C,
    0xAA,0xD1,0xEE,0xEE,0xDF,0x0F,0x75,0xBC,0x3F,0xBF,0xCD,0x9C,0xAB,0x3D,0xE9,0x72
])

FACTORY_PAIRING_PUBLIC_KEY_PROD0 = bytes([
    0xF9,0x75,0xEB,0x3C,0x2F,0xD7,0x90,0xC9,0x6F,0x29,0x4F,0x15,0x57,0xA5,0x03,0x17,
    0x80,0xC9,0xAA,0xFA,0x14,0x0D,0xA2,0x8F,0x55,0xE7,0x51,0x57,0x37,0xB2,0x50,0x2C
])
