RSP_STATUS_REQ_OK = 0x01
RSP_STATUS_RES_OK = 0x02
RSP_STATUS_REQ_CONT = 0x03
RSP_STATUS_RES_CONT = 0x04
RSP_STATUS_RESP_DISABLED = 0x78
RSP_STATUS_HSK_ERROR = 0x79
RSP_STATUS_NO_SESSION = 0x7A
RSP_STATUS_TAG_ERROR = 0x7B
RSP_STATUS_CRC_ERROR = 0x7C
RSP_STATUS_UNKNOWN_REQ = 0x7E
RSP_STATUS_GEN_ERROR = 0x7F
RSP_STATUS_NO_RESPONSE = 0xFF
