"""Config registry addresses - public API

These constants represent memory-mapped configuration registers
in the TROPIC01 secure element. Use them with r_config_read(),
i_config_read(), and parse_config().
"""

# Bootloader configuration registers
CFG_START_UP = 0x00
CFG_SENSORS = 0x08
CFG_DEBUG = 0x10

# Application configuration registers
CFG_GPO = 0x14
CFG_SLEEP_MODE = 0x18
CFG_UAP_PAIRING_KEY_WRITE = 0x20
CFG_UAP_PAIRING_KEY_READ = 0x24
CFG_UAP_PAIRING_KEY_INVALIDATE = 0x28
CFG_UAP_R_CONFIG_WRITE_ERASE = 0x30
CFG_UAP_R_CONFIG_READ = 0x34
CFG_UAP_I_CONFIG_WRITE = 0x40
CFG_UAP_I_CONFIG_READ = 0x44
CFG_UAP_PING = 0x100
CFG_UAP_R_MEM_DATA_WRITE = 0x110
CFG_UAP_R_MEM_DATA_READ = 0x114
CFG_UAP_R_MEM_DATA_ERASE = 0x118
CFG_UAP_RANDOM_VALUE_GET = 0x120
CFG_UAP_ECC_KEY_GENERATE = 0x130
CFG_UAP_ECC_KEY_STORE = 0x134
CFG_UAP_ECC_KEY_READ = 0x138
CFG_UAP_ECC_KEY_ERASE = 0x13C
CFG_UAP_ECDSA_SIGN = 0x140
CFG_UAP_EDDSA_SIGN = 0x144
CFG_UAP_MCOUNTER_INIT = 0x150
CFG_UAP_MCOUNTER_GET = 0x154
CFG_UAP_MCOUNTER_UPDATE = 0x158
CFG_UAP_MAC_AND_DESTROY = 0x160
